package com.flyfish.framework.compiler;

import com.flyfish.framework.compiler.support.DelegateJavaCompiler;
import com.flyfish.framework.compiler.support.SimpleJavaCompiler;

import java.io.Closeable;
import java.io.IOException;

/**
 * 动态的java编译器
 * 提供动态的java编译
 *
 * @author wangyu
 */
public interface DynamicJavaCompiler extends Closeable {

    /**
     * 代理的java编译器，使用库文件
     *
     * @return 结果
     */
    static DynamicJavaCompiler delegate() {
        return DelegateJavaCompiler.getInstance();
    }

    /**
     * 简单的java编译器
     *
     * @return 结果
     */
    static DynamicJavaCompiler simple() {
        return SimpleJavaCompiler.getInstance();
    }

    /**
     * 编译java源码，通过字符串
     *
     * @param name   文件名
     * @param source 源码
     * @return 结果
     */
    Class<?> compile(String name, String source) throws ClassNotFoundException, IOException;
}
