package com.flyfish.framework.service;


import com.flyfish.framework.domain.PermissionQo;
import com.flyfish.framework.domain.po.Permission;
import com.flyfish.framework.domain.po.Role;
import com.flyfish.framework.enums.RoleType;
import com.flyfish.framework.service.impl.BaseReactiveServiceImpl;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.BooleanUtils;
import org.springframework.stereotype.Service;
import reactor.core.publisher.Mono;

import java.util.Collections;

@Service
@RequiredArgsConstructor
public class RoleService extends BaseReactiveServiceImpl<Role> {

    private final PermissionService permissionService;

    /**
     * 如果是管理员，设置拥有所有权限
     *
     * @param role 实体
     * @return 结果
     */
    @Override
    public Mono<Role> create(Role role) {
        return Mono.defer(() -> {
                    if (role.isSystem()) {
                        return permissionService.getAll().collectList();
                    } else if (BooleanUtils.isTrue(role.getAdmin())) {
                        // 如果是管理员，拥有所有权限
                        PermissionQo qo = new PermissionQo();
                        qo.setAdmin(false);
                        return permissionService.getList(qo).collectList();
                    }
                    return Mono.just(Collections.<Permission>emptyList());
                })
                .flatMap(permissions -> {
                    role.setPermissions(permissions);
                    if (null == role.getType()) {
                        role.setType(RoleType.PC);
                    }
                    return super.create(role);
                });
    }
}
