package com.flyfish.framework.beans.meta.parser.chain;

import java.lang.annotation.Annotation;
import java.util.List;
import java.util.function.Consumer;
import java.util.function.Function;
import java.util.function.Predicate;
import java.util.function.Supplier;

/**
 * 抽象的链式操作，作为批量支持
 *
 * @param <A> 注解泛型
 * @param <T> 入参类型
 */
public interface BeanPropertyAnnotationBatchChain<A extends Annotation, T> extends AnnotationChainSupport {

    /**
     * 映射转换，将转换每一条
     *
     * @param mapper 映射器
     * @param <R>    泛型
     * @return 结果
     */
    <R> BeanPropertyAnnotationBatchChain<A, R> map(Function<T, R> mapper);

    /**
     * 过滤，不满足的将过滤，如果为空，将empty
     *
     * @param filter 过滤器
     * @return 结果
     */
    BeanPropertyAnnotationBatchChain<A, T> filter(Predicate<T> filter);

    /**
     * 过滤，不满足的将过滤，如果为空，将empty
     *
     * @param filter 过滤器
     * @return 结果
     */
    BeanPropertyAnnotationBatchChain<A, T> filter(Supplier<Boolean> filter);

    /**
     * 如果存在，则做
     *
     * @param consumer 消费者
     * @return 结果
     */
    BeanPropertyAnnotationBatchChain<A, T> then(Consumer<List<T>> consumer);

    /**
     * 存在时执行逻辑，不需要结果
     *
     * @param handler 处理器
     * @return 结果
     */
    BeanPropertyAnnotationBatchChain<A, T> exists(Runnable handler);

    /**
     * 不存在做的事情
     *
     * @param fallback 补偿逻辑
     * @return 结果
     */
    BeanPropertyAnnotationBatchChain<A, T> empty(Runnable fallback);

    /**
     * 获取所有结果为list
     *
     * @return 结果
     */
    List<A> synthesize();


}
