package com.flyfish.framework.configuration.boot;

import com.flyfish.framework.annotations.EnableMongoRepo;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.context.annotation.EnableAspectJAutoProxy;
import org.springframework.core.annotation.AliasFor;
import org.springframework.scheduling.annotation.EnableAsync;

import java.lang.annotation.*;

/**
 * 飞鱼便捷入口，一个注解，所有工作
 * All in One 小霸王注解，注意指定扫描路径
 *
 * @author wangyu
 */
@Target({ElementType.TYPE})
@Retention(RetentionPolicy.RUNTIME)
@Documented
@Inherited
@EnableMongoRepo
@SpringBootApplication
@EnableAsync(proxyTargetClass = true)
@EnableAspectJAutoProxy
public @interface FlyfishApplication {

    /**
     * repo扫描的基础包
     *
     * @return 结果
     */
    @AliasFor(
            annotation = EnableMongoRepo.class,
            attribute = "basePackages"
    )
    String[] repositoryBasePackages() default {};

    /**
     * 扫描组件的路径
     *
     * @return 结果
     */
    @AliasFor(
            annotation = SpringBootApplication.class
    )
    String[] scanBasePackages() default "com.flyfish";
}
