package com.flyfish.framework.configuration.resolver;

import com.flyfish.framework.configuration.annotations.RequestContextBody;
import com.flyfish.framework.domain.base.Domain;
import com.flyfish.framework.utils.UserUtils;
import org.springframework.core.MethodParameter;
import org.springframework.core.ReactiveAdapterRegistry;
import org.springframework.http.codec.HttpMessageReader;
import org.springframework.security.core.context.ReactiveSecurityContextHolder;
import org.springframework.util.Assert;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.reactive.BindingContext;
import org.springframework.web.reactive.result.method.annotation.AbstractMessageReaderArgumentResolver;
import org.springframework.web.server.ServerWebExchange;
import reactor.core.publisher.Mono;

import java.util.List;

/**
 * 附带上下文里的请求参数解析器
 *
 * @author wangyu
 */
public class RequestContextBodyArgumentResolver extends AbstractMessageReaderArgumentResolver {

    public RequestContextBodyArgumentResolver(List<HttpMessageReader<?>> readers,
                                              ReactiveAdapterRegistry registry) {
        super(readers, registry);
    }


    @Override
    public boolean supportsParameter(MethodParameter parameter) {
        return parameter.hasParameterAnnotation(RequestContextBody.class);
    }

    @Override
    public Mono<Object> resolveArgument(
            MethodParameter param, BindingContext bindingContext, ServerWebExchange exchange) {
        RequestContextBody ann = param.getParameterAnnotation(RequestContextBody.class);
        Assert.state(ann != null, "No RequestBody annotation");
        return readBody(param, ann.required(), bindingContext, exchange)
                .flatMap(object -> {
                    if (object instanceof Domain) {
                        return ReactiveSecurityContextHolder.getContext()
                                .map(UserUtils::extractUser)
                                .map(user -> {
                                    Domain entity = (Domain) object;
                                    entity.setCurrentUser(user);
                                    return object;
                                })
                                .defaultIfEmpty(object);
                    }
                    return Mono.just(object);
                });
    }

}
